/*
  Program:      Mine Mayhem
  File:         options.cc
  Date: 	July, 1996 to February,   1997, version 1.00.
		26 October to 3 December,	version 2.00.
		8 December to 4 January,  2001, version 2.10.
		15 January,			version 2.11.

  The configuration dialog box.

  Freeware, Copyright 2001 Jason Hood.

  You are free to use this code, or a portion thereof, as long as an
  appropriate acknowledgement is made.

  v2.10: moved Game Type to the menu;
	 added auto-clear and tracking options.
*/

#include "options.h"
#include "minefld.h"

#define ID_CHECK_FINOPEN   1000
#define ID_CHECK_FINMARK   1001

#define ID_CHECK_MARKS	   1002
#define ID_CHECK_WRAP	   1003
#define ID_CHECK_TRACK	   1004

#define ID_CHECK_AUTOMARK  1005
#define ID_CHECK_AUTOOPEN  1006
#define ID_CHECK_AUTOCLEAR 1007


DEFINE_RESPONSE_TABLE(OptionDlg, TabWin)
   E_BUTTONUP(ID_OK, cmOK)
   E_BUTTONUP(ID_CANCEL, cmCancel)
END_RESPONSE_TABLE

#define DLG_WIDTH  (SysFont->width*48)
#define DLG_HEIGHT (SysFont->height*17)

OptionDlg::OptionDlg() :
  TabWin(NULL, "Options",
         (ws.GetDeskWidth()-DLG_WIDTH-FWIDTH*2)/2,
         (ws.GetDeskHeight()-DLG_HEIGHT-FWIDTH-CAPTION_HEIGHT)/2,
         (ws.GetDeskWidth()+DLG_WIDTH+FWIDTH*2)/2-1,
         (ws.GetDeskHeight()+DLG_HEIGHT+FWIDTH+CAPTION_HEIGHT)/2,
         WA_VISABLE | WA_BORDER | WA_CAPTION | WA_SAVEAREA),
  isOK(false)
{
  int ox, oy,
      wid = 20*SysFont->width+8, hyt = 6 + 3*(SysFont->height+1);

  ox = FWIDTH+8; oy = FWIDTH+CAPTION_HEIGHT+8+SysFont->height+2;
  start = new RadioButtonContainer(this, ox, oy, ox+wid-1, oy+hyt-1,
                                   RADIO_TYPE_DIP);
  start->AddButton("&No repositioning", NO_REPOS);
  start->AddButton("One &displacement", ONE_REPOS);
  start->AddButton("&Automatic", AUTO);
  start->SelectButton(Start);

  wid -= 20;
  ox += 4; oy += hyt+SysFont->height*2+2+3;
  finishopen = new CheckBox(this, ID_CHECK_FINOPEN, "&Opened", ox, oy, wid,
                            (Finish & ALL_OPENED) != 0);
  oy += SysFont->height+1;
  finishmark = new CheckBox(this, ID_CHECK_FINMARK, "&Marked", ox, oy, wid,
                            (Finish & ALL_MARKED) != 0);

  wid += 20;
  ox -= 4; oy += SysFont->height+3+SysFont->height+SysFont->height+2;
  int boy = oy;
  clear = new RadioButtonContainer(this, ox, oy, ox+wid-1, oy+hyt-1,
                                   RADIO_TYPE_DIP);
  clear->AddButton("&Left", CLRLEFT);
  clear->AddButton("&Right", CLRRIGHT);
  clear->AddButton("M&iddle", CLRMIDDLE);
  clear->SelectButton(ClearButton);

  ox = DLG_WIDTH - FWIDTH-8 - wid + 4;
  oy = FWIDTH+CAPTION_HEIGHT+8+SysFont->height+2+3;
  wid -= 20;
  marks = new CheckBox(this, ID_CHECK_MARKS, "Marks (&?)", ox,oy, wid, isMarks);
  AddAccelerator('/', marks);
  oy += SysFont->height+1;
  wrap = new CheckBox(this, ID_CHECK_WRAP, "&Wrap-around", ox, oy, wid, isWrap);
  oy += SysFont->height+1;
  track = new CheckBox(this, ID_CHECK_TRACK, "&Tracking", ox,oy,wid, isTracked);

  oy += SysFont->height+3+SysFont->height+SysFont->height+2+3;
  automark = new CheckBox(this, ID_CHECK_AUTOMARK, "Markin&g",
                          ox, oy, wid, isAutoMark);
  oy += SysFont->height+1;
  autoopen = new CheckBox(this, ID_CHECK_AUTOOPEN, "O&pening",
                          ox, oy, wid, isAutoOpen);
  oy += SysFont->height+1;
  autoclear = new CheckBox(this, ID_CHECK_AUTOOPEN, "Cl&earing",
			   ox, oy, wid, isAutoClear);

  int bwid = 10*SysFont->width, bhyt = SysFontBold->height+4;
  ox += (wid+20) - bwid - 4;
  oy = boy + (hyt - (bhyt*2+4));
  OK = new TextButton(this, ID_OK, ox, oy, ox+bwid-1, oy+bhyt-1, "O&K");
  oy += bhyt + 4;
  Cancel = new TextButton(this, ID_CANCEL, ox, oy,
			  ox+bwid-1, oy+bhyt-1, "&Cancel");

  AddTabStop(start);
  AddTabStop(finishopen);
  AddTabStop(finishmark);
  AddTabStop(marks);
  AddTabStop(wrap);
  AddTabStop(track);
  AddTabStop(automark);
  AddTabStop(autoopen);
  AddTabStop(autoclear);
  AddTabStop(clear);
  AddTabStop(OK);
  AddTabStop(Cancel);
}


OptionDlg::~OptionDlg()
{
  delete Cancel; delete OK;
  delete start;
  delete finishopen; delete finishmark;
  delete clear;
  delete marks; delete wrap; delete track;
  delete automark; delete autoopen; delete autoclear;
}

extern BYTE HighLightColour;

// Write all the headings and draw the boxes around the options.
void OptionDlg::PaintWindow(int x1, int y1, int x2, int y2)
{
  TabWin::PaintWindow(x1, y1, x2, y2);

  ViewBuffer* buf = GetSubBuffer(x1, y1, x2, y2);

  int ox = FWIDTH+8, oy = FWIDTH+CAPTION_HEIGHT+8,
      wid = 20*SysFont->width+8,
      hyt = 3 + 3*(SysFont->height+1) + 3;

  WriteText(buf, ox,oy, HighLightColour, SysFontItallic, "Starting Condition");
  oy += SysFont->height+2 + hyt + SysFont->height;
  WriteText(buf, ox,oy, HighLightColour, SysFontItallic, "Finishing Condition");

  oy += SysFont->height+2;
  hyt -= SysFont->height+1;
  InvFrame(buf, ox, oy, ox+wid-1, oy+hyt-1);
     Frame(buf, ox+1, oy+1, ox+wid-2, oy+hyt-2);

  oy += hyt + SysFont->height;
  WriteText(buf, ox,oy, HighLightColour, SysFontItallic, "Clear-Around Button");

  ox = DLG_WIDTH - (wid+8+FWIDTH); oy = FWIDTH+CAPTION_HEIGHT+8;
  WriteText(buf, ox, oy, HighLightColour, SysFontItallic, "General");
  oy += SysFont->height+2;
  hyt += SysFont->height+1;
  InvFrame(buf, ox, oy, ox+wid-1, oy+hyt-1);
     Frame(buf, ox+1, oy+1, ox+wid-2, oy+hyt-2);

  oy += hyt + SysFont->height;
  WriteText(buf, ox,oy, HighLightColour, SysFontItallic, "Automatic");
  oy += SysFont->height+2;
  InvFrame(buf, ox, oy, ox+wid-1, oy+hyt-1);
     Frame(buf, ox+1, oy+1, ox+wid-2, oy+hyt-2);

  DeleteBuffer(buf);
}


void OptionDlg::Run()
{
  Paint();
  RefreshWindow();
  SetFocus(OK);
  CatchMouse();
  CatchKeys();
  ws.RunEvents();
  ws.ResetEvents();
  ReleaseMouse();
  ReleaseKeys();
  if (isOK)
  {
    if (!field->GameinProgress())
    {
      isWrap = wrap->isChecked();
      field->AdjustWrap();		// Make sure the edges are right
      Start  = start->GetSelectedButton();
      Finish = finishopen->isChecked() | (finishmark->isChecked() << 1);
      isAutoMark = automark->isChecked();
      isAutoOpen = autoopen->isChecked();
    }
    isMarks = marks->isChecked();
    isTracked = track->isChecked();
    isAutoClear = autoclear->isChecked();
    ClearButton = clear->GetSelectedButton();
  }
}

void OptionDlg::cmOK()
{
  isOK = true;
  ws.StopRunningEvents();
}

void OptionDlg::cmCancel()
{
  isOK = false;
  ws.StopRunningEvents();
}
